// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Net.Imap4.Protocol;
using Smdn.Text;

namespace Smdn.Net.Imap4 {
  public class ImapExtendedMultiPartBodyStructure : ImapMultiPartBodyStructure, IImapBodyStructureExtension {
    public IReadOnlyDictionary<string, string> Parameters {
      get; private set;
    }

    public ImapBodyDisposition Disposition {
      get; private set;
    }

    public IReadOnlyList<string> Languages {
      get; private set;
    }

    public Uri Location {
      get; private set;
    }

    public IReadOnlyList<ImapData> Extensions {
      get; private set;
    }

    public ImapExtendedMultiPartBodyStructure
    (
      ImapMultiPartBodyStructure baseStructure,
      IReadOnlyDictionary<string, string> parameters,
      ImapBodyDisposition disposition,
      IReadOnlyList<string> languages,
      Uri location,
      IReadOnlyList<ImapData> extensions
    )
      : base(baseStructure)
    {
      this.Parameters = CollectionUtils.ToNonNullReadOnly(parameters);
      this.Disposition = disposition;
      this.Languages = CollectionUtils.ToNonNullReadOnly(languages);
      this.Location = location;
      this.Extensions = CollectionUtils.ToNonNullReadOnly(extensions);
    }

    public override string ToString()
    {
      return string.Format("{{Parameters={{{0}}}, Disposition={1}, Languages={{{2}}}, Location={3}, Extensions={{{4}}}, BaseStructure={{{5}}}}}",
                           StringConversion.ToJoinedString(Parameters),
                           Disposition,
                           string.Join(", ", Languages),
                           Location,
                           string.Join(", ", Extensions),
                           base.ToString());
    }
  }
}
