// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;

namespace Smdn.Net.Imap4 {
  public sealed class ImapNamespace
#if NETFRAMEWORK || NETSTANDARD2_0
    : ICloneable
#endif
  {
    public IReadOnlyList<ImapNamespaceDesc> PersonalNamespaces {
      get; private set;
    }

    public IReadOnlyList<ImapNamespaceDesc> OtherUsersNamespaces {
      get; private set;
    }

    public IReadOnlyList<ImapNamespaceDesc> SharedNamespaces {
      get; private set;
    }

    public ImapNamespace()
      : this(null, null, null)
    {
    }

    public ImapNamespace
    (
      IReadOnlyList<ImapNamespaceDesc> personalNamespaces,
      IReadOnlyList<ImapNamespaceDesc> otherUsersNamespaces,
      IReadOnlyList<ImapNamespaceDesc> sharedNamespaces
    )
    {
      this.PersonalNamespaces   = CollectionUtils.ToNonNullReadOnly(personalNamespaces);
      this.OtherUsersNamespaces = CollectionUtils.ToNonNullReadOnly(otherUsersNamespaces);
      this.SharedNamespaces     = CollectionUtils.ToNonNullReadOnly(sharedNamespaces);
    }

#if NETFRAMEWORK || NETSTANDARD2_0
    object ICloneable.Clone()
    {
      return this.Clone();
    }
#endif

    public ImapNamespace Clone()
    {
      return new ImapNamespace(PersonalNamespaces.Select((Func<ImapNamespaceDesc, ImapNamespaceDesc>)CloneNamespaceDesc).ToList(),
                               OtherUsersNamespaces.Select((Func<ImapNamespaceDesc, ImapNamespaceDesc>)CloneNamespaceDesc).ToList(),
                               SharedNamespaces.Select((Func<ImapNamespaceDesc, ImapNamespaceDesc>)CloneNamespaceDesc).ToList());
    }

    private static ImapNamespaceDesc CloneNamespaceDesc(ImapNamespaceDesc desc)
    {
      return desc.Clone();
    }

    public override string ToString()
    {
      return string.Format("{{PersonalNamespaces={{{0}}}, OtherUsersNamespaces={{{1}}}, SharedNamespaces={{{2}}}}}",
                           string.Join(", ", PersonalNamespaces),
                           string.Join(", ", OtherUsersNamespaces),
                           string.Join(", ", SharedNamespaces));
    }
  }
}
