// 詳細なドキュメントおよびサンプルは以下のページをご覧ください
// https://smdn.jp/works/libs/Smdn.Net.Imap4.Client/docs/

using System;
using System.Security.Authentication;
using Smdn.Net.Imap4.Client;

class MainClass {
  static void Main(string[] args)
  {
    // SSL/TLSでの接続を行う場合は、証明書の検証を行うように書き換えてください
    ImapSslConnection.ServerCertificateValidationCallback = ImapSslConnection.DangerousAcceptAnyServerCertificateValidator;

    // 有効にするSSL/TLSプロトコルバージョンを指定する場合は、SslProtocolsの値を変更してください
    ImapSslConnection.EnabledSslProtocols = SslProtocols.None;

    const string hostname = "imap.example.net";
    const string username = "user";
    const string password = "pass";

    using (var client = new ImapClient(new Uri($"imap://{username}@{hostname}/"))) {
      client.Profile.UseTlsIfAvailable = true;    // 可能ならSSL/TLSでの接続を行う
      client.Profile.AllowInsecureLogin = false;  // 平文での認証を許可しない

      // IMAPサーバーへの接続を行う
      client.Connect(password);

      Console.WriteLine("connected!!");

      // すべてのメールボックスとメールボックスのステータスを取得して列挙
      foreach (var mailbox in client.GetMailboxes(ImapMailboxListOptions.RequestStatus)) {
        // 合計メッセージ数と名前を表示
        Console.WriteLine("{0} ({1})", mailbox.FullName, mailbox.ExistMessageCount);
      }

      // INBOXを開く
      using (var inbox = client.OpenInbox()) {
        // 合計・未読・新着メッセージ数
        Console.WriteLine("[INBOX]");
        Console.WriteLine("total message count : {0}", inbox.ExistMessageCount);
        Console.WriteLine("unseen message count : {0}", inbox.UnseenMessageCount);
        Console.WriteLine("recent message count : {0}", inbox.RecentMessageCount);

        // すべてのメッセージとその属性を取得
        foreach (var message in inbox.GetMessages(ImapMessageFetchAttributeOptions.AllAttributes)) {
          if (message.IsRecent) // 新着
            Console.Write("!");
          else if (!message.IsSeen) // 未読
            Console.Write("*");
          else
            Console.Write(" ");

          // 着信日時とタイトルを表示
          Console.WriteLine(" ({0:o}) {1}", message.DeliveredDate, message.EnvelopeSubject);
        }
      }
    }
  }
}
